from xperms import get_perm_model
from django.test import TestCase, override_settings

from .base import ArticleUserPermTestCase, ArticleGroupPermTestCase


Perm = get_perm_model()


class GenericPermTestCaseMixin:

    def _create_perm(self):
        return Perm.objects.create(
            codename='export',
        )

    def _create_perm_wildcard(self):
        return Perm.objects.create(
            codename='*',
        )


class GenericPermTestCase(GenericPermTestCaseMixin, TestCase):

    def test_perm_has_correct_type(self):
        perm = self._create_perm()
        self.assertTrue(perm.is_generic_perm())


class ArticleUserGenericPermPermTestCase(GenericPermTestCaseMixin, ArticleUserPermTestCase):

    def test_add_generic_perm_by_perm(self):
        perm = self._create_perm()

        self.user.perms.add_perm(perm)

        self.assertTrue(self.user.perms.has_perm(perm))

    def test_add_generic_perm_by_codename(self):
        export_perm = self._create_perm()

        self.user.perms.add_perm('generic.export')

        self.assertTrue(self.user.perms.has_perm(export_perm))

    def test_fail_add_generic_perm_non_existent_codename(self):
        with self.assertRaises(Perm.DoesNotExist):
            self.user.perms.add_perm('generic.export')

    @override_settings(PERM_AUTO_CREATE=True)
    def test_auto_create_generic_perm(self):
        self.user.perms.add_perm('generic.export')

    @override_settings(PERM_AUTO_CREATE=True)
    def test_clear_perms(self):
        self.user.perms.add_perm('generic.export')
        self.user.perms.add_perm('generic.foo')
        self.user.perms.add_perm('generic.bar')
        self.user.perms.add_perm('generic.baz')

        self.assertEquals(self.user.perms.all().count(), 4)

        self.user.perms.clear()

        self.assertEquals(self.user.perms.all().count(), 0)

    @override_settings(PERM_AUTO_CREATE=True)
    def test_remove_generic_perm_by_codename(self):
        self.user.perms.add_perm('generic.export')
        self.assertTrue(self.user.perms.has_perm('generic.export'))

        self.user.perms.remove_perm('generic.export')
        self.assertFalse(self.user.perms.has_perm('generic.export'))

    @override_settings(PERM_AUTO_CREATE=True)
    def test_remove_generic_perm_by_perm(self):
        export_perm = self._create_perm()

        self.user.perms.add_perm(export_perm)
        self.assertTrue(self.user.perms.has_perm(export_perm))

        self.user.perms.remove_perm(export_perm)
        self.assertFalse(self.user.perms.has_perm(export_perm))

    def test_has_generic_perm_from_wildcard(self):
        self._create_perm_wildcard()

        self.user.perms.add_perm('generic.*')

        self.assertTrue(self.user.perms.has_perm('generic.whatever'))

    def test_has_generic_perm_from_being_superuser(self):
        self._create_perm_wildcard()

        self.user.is_superuser = True
        self.user.save()

        self.assertTrue(self.user.perms.has_perm('generic.whatever'))


class ArticleGroupGenericPermPermTestCase(GenericPermTestCaseMixin, ArticleGroupPermTestCase):

    def test_add_generic_perm_by_perm(self):
        perm = self._create_perm()

        self.group.perms.add_perm(perm)

        # test the new group perm is the created generic perm
        self.assertTrue(self.group.perms.has_perm(perm))

    def test_add_generic_perm_by_codename(self):
        export_perm = self._create_perm()

        self.group.perms.add_perm('generic.export')

        self.assertTrue(self.group.perms.has_perm(export_perm))

        # test perm is correctly available to the user as well
        self.assertFalse(self.user.perms.has_perm(export_perm))

        self.user.groups.add(self.group)

        self.assertTrue(self.user.perms.has_perm(export_perm))

    def test_fail_add_generic_perm_non_existent_codename(self):
        with self.assertRaises(Perm.DoesNotExist):
            self.group.perms.add_perm('generic.export')



from xperms import enums
from xperms.models import Perm

from .base import ArticleTestCase, ArticleUserPermTestCase, ArticleGroupPermTestCase


class FieldPermTestCaseMixin:

    def _create_perm(self):
        return self._create_add_perm()

    def _create_add_perm(self):
        return Perm.objects.create(
            type=enums.PERM_TYPE_FIELD,
            codename=enums.PERM_CODENAME_ADD,
            content_type=self._get_content_type(),
            field_name='name',
        )

    def _create_delete_perm(self):
        return Perm.objects.create(
            type=enums.PERM_TYPE_FIELD,
            codename=enums.PERM_CODENAME_DELETE,
            content_type=self._get_content_type(),
            field_name='name',
        )

    def _create_wildcard_perm(self):
        return Perm.objects.create(
            type=enums.PERM_TYPE_FIELD,
            codename=enums.PERM_CODENAME_WILDCARD,
            content_type=self._get_content_type(),
            field_name='name',
        )


class FieldPermTestCase(FieldPermTestCaseMixin, ArticleTestCase):

    def test_perm_has_correct_type(self):
        perm = self._create_perm()
        self.assertTrue(perm.is_field_perm())


class ArticleUserFieldPermPermTestCase(FieldPermTestCaseMixin, ArticleUserPermTestCase):

    def test_add_field_perm_by_perm(self):
        perm = self._create_perm()

        self.user.perms.add_perm(perm)

        self.user.perms.has_perm(perm)

    def test_add_field_perm_by_str(self):
        add_name_perm = self._create_add_perm()

        self.user.perms.add_perm('field.articles.Article.name.add')

        self.user.perms.has_perm(add_name_perm)

    def test_fail_add_field_perm_non_existent_codename(self):
        self._create_perm()
        with self.assertRaises(Perm.DoesNotExist):
            self.user.perms.add_perm('field.articles.Article.name.delete')

    def test_fail_add_field_perm_non_existent_model(self):
        self._create_perm()
        with self.assertRaises(LookupError):
            self.user.perms.add_perm('field.articles.Foo.name.add')

    def test_fail_add_field_perm_non_existent_field_name(self):
        self._create_perm()
        with self.assertRaises(Perm.DoesNotExist):
            self.user.perms.add_perm('field.articles.Article.foo.add')

    def test_has_field_perm_from_wildcard(self):
        self._create_wildcard_perm()

        self.user.perms.add_perm('field.articles.Article.name.*')

        self.assertTrue(self.user.perms.has_perm('field.articles.Article.name.whatever'))

    def test_fail_has_perm_non_existent_field_name(self):
        add_name_perm = self._create_add_perm()

        self.user.perms.add_perm(add_name_perm)

        self.assertFalse(self.user.perms.has_perm('field.articles.Article.foobar.add'))

    def test_fail_has_perm_non_existent_codename(self):
        add_name_perm = self._create_add_perm()

        self.user.perms.add_perm(add_name_perm)

        self.assertFalse(self.user.perms.has_perm('field.articles.Article.name.delete'))


class ArticleGroupFieldPermPermTestCase(FieldPermTestCaseMixin, ArticleGroupPermTestCase):

    def test_add_field_perm_by_perm(self):
        perm = self._create_perm()

        self.group.perms.add_perm(perm)

        self.group.perms.has_perm(perm)

    def test_add_field_perm_by_str(self):
        add_name_perm = self._create_add_perm()

        self.group.perms.add_perm('field.articles.Article.name.add')

        self.assertTrue(self.group.perms.has_perm(add_name_perm))

        # test perm is correctly available to the user as well
        self.assertFalse(self.user.perms.has_perm(add_name_perm))

        self.user.groups.add(self.group)

        self.assertTrue(self.user.perms.has_perm(add_name_perm))

    def test_fail_add_field_perm_non_existent_codename(self):
        self._create_perm()
        with self.assertRaises(Perm.DoesNotExist):
            self.group.perms.add_perm('field.articles.Article.name.delete')

    def test_fail_add_field_perm_non_existent_model(self):
        self._create_perm()
        with self.assertRaises(LookupError):
            self.group.perms.add_perm('field.articles.Foo.name.add')

    def test_fail_add_field_perm_non_existent_field_name(self):
        self._create_perm()
        with self.assertRaises(Perm.DoesNotExist):
            self.group.perms.add_perm('field.articles.Article.foo.add')



__version__ = '0.4.17'
default_app_config = 'xperms.apps.XPermsConfig'

from django.apps import apps as django_apps
from xperms.conf import settings

from xperms.exceptions import ImproperlyConfigured


def get_perm_model():
    """
    Returns the Perm model that is active in this project.
    """
    try:
        return django_apps.get_model(settings.PERM_MODEL, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured("PERM_MODEL must be of the form 'app_label.model_name'")
    except LookupError:
        raise ImproperlyConfigured(
            "PERM_MODEL refers to model '{}' that has not been installed".format(settings.PERM_MODEL)
        )



from django.contrib.auth import get_user_model

import factory
from factory import fuzzy

User = get_user_model()


DEFAULT_PASSWORD = 'foofoobar'


class UserFactory(factory.DjangoModelFactory):

    username = factory.LazyAttribute(lambda a: '{0}'.format(a.last_name).lower())

    first_name = factory.Sequence(lambda n: 'John{0}'.format(n))
    last_name = factory.Sequence(lambda n: 'Doe{0}'.format(n))
    email = factory.LazyAttribute(lambda a: '{0}.{1}@example.com'.format(a.first_name, a.last_name).lower())
    password = DEFAULT_PASSWORD

    is_active = True

    class Meta:
        model = User
        django_get_or_create = ('username',)


class GroupFactory(factory.DjangoModelFactory):

    name = factory.Sequence(lambda n: 'Group{0}'.format(n))

    class Meta:
        model = 'auth.Group'


class ArticleFactory(factory.DjangoModelFactory):

    name = fuzzy.FuzzyText()
    text = fuzzy.FuzzyText()

    class Meta:
        model = 'articles.Article'



class PermError(Exception):
    pass


class ObjectNotPersisted(PermError):
    pass


class IncorrectObject(PermError):
    pass


class IncorrectContentType(PermError):
    pass


class ImproperlyConfigured(PermError):
    pass



# Generated by Django 4.1 on 2022-08-11 14:10

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Perm',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[('generic', 'generic'), ('model', 'model'), ('object', 'object'), ('field', 'field')], default='generic', max_length=10)),
                ('codename', models.CharField(max_length=100, verbose_name='codename')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='name')),
                ('object_id', models.TextField(blank=True, null=True, verbose_name='object pk')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='content type')),
                ('users', models.ManyToManyField(blank=True, related_name='xperms', to=settings.AUTH_USER_MODEL, verbose_name='users')),
            ],
            options={
                'base_manager_name': 'related_manager',
            },
            managers=[
                ('related_manager', django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('codename', models.CharField(blank=True, max_length=100, null=True, unique=True, verbose_name='codename')),
                ('name', models.CharField(max_length=255, verbose_name='name')),
                ('description', models.CharField(blank=True, default='', max_length=180)),
                ('users', models.ManyToManyField(blank=True, related_name='xgroups', to=settings.AUTH_USER_MODEL, verbose_name='users')),
                ('xgroups', models.ManyToManyField(blank=True, related_name='parents', to='xperms.group', verbose_name='subgroups')),
                ('xperms', models.ManyToManyField(blank=True, related_name='xgroups', to='xperms.perm', verbose_name='permissions')),
            ],
            options={
                'verbose_name': 'permission group',
                'verbose_name_plural': 'permission groups',
                'ordering': ('codename',),
            },
        ),
    ]
